#!/bin/sh
#
#        VideoEdge IPSpeedDome Firmware Upgrade
#
#          Copyright 2008 American Dynamics
#

###################### Functions ########################################

currentfirmware=`/opt/ad/bin/domeinfo | awk '{print$6}'`
installer=`basename $0`

fatal()
{
    local result=$?
    echo Error: $* 1>&2
    logger -s -t $installer -p user.err "Error: $*"
    exit $result
}

warning()
{
    echo Warning: $*
}

log()
{
    logger -s -t $installer  "$*"
}

# YT 20120606: this should now be copy from one patition to other
extract_backups()
{
    local upgrade_tar=$1
    tar -xzvpOf ${upgrade_tar:?undefined} inner.tar.gz | tar zpvxf - -C /  \
      opt/ad/bin/config_backup.sh \
      opt/ad/bin/camerablocksettings \
      opt/ad/bin/config_encrypt.sh \
      opt/ad/etc/conf/CONFIGBK.conf \
      opt/ad/etc/conf/CONFIGPW.conf || fatal "restore of backup scripts failed"
   echo backup scripts restored
}

# YT20120606 for future use
check_md5()
{
# check md5
	echo "extracting inner tarball md5 checksum"
    tar -C /tmp/ -xpvzf $firmware inner.tar.gz.md5 2>&1 || fatal "unable to extract md5 checksum"

	sample1=`tar -xzOvf $firmware inner.tar.gz|md5sum|head -c 32`
	sample2=`cat /tmp/inner.tar.gz.md5|head -c 32`

	echo "checking md5"
	echo "sample1:$sample1 sample2:$sample2"

   if [ $sample1 != $sample2 ]; then
    	fatal "md5 checksums do not match"
    fi
    echo "md5 checksums match"
}


merge_files()
{
	# merge conf files
	if [ -f /mnt/flash/opt/ad/bin/config_merge.sh ] ; then
		/mnt/flash/opt/ad/bin/config_merge.sh
	else
		echo "config_merge.sh file does not exist"
	fi
}

copy_config()
{
	############################################################
	# Pre-Upgrade phase (delete old files)
	############################################################

	# Extract upgrade versions of backup utilities
	# No longer necessary as we run from new partition
	# extract_backups $firmware
	# Ensure camera block settings are stored correctly
	camblkrestorerevision=25 # ipdome.1.0.25.xxx.tar.gz
	if [ "$currentrevision" -lt "$camblkrestorerevision" -a "$currentmajor" -eq "1" -a "$currentminor" -eq "0" ]; then
	    # Remove old CAMBLK*.conf files if present, keeping only CAMBLK_X22.conf, CAMBLK_X35.conf and CAMBLK_X35_N.conf
	    #| egrep -v "CAMBLK_X22\.conf|CAMBLK_X35\.conf|CAMBLK_X35_N\.conf"
	    # removed match to these files as they were gen 1 only
	    find /opt/ad/etc/conf -name "CAMBLK*.conf" | xargs rm -f
	fi

	# Write current PTZ position to position.dat file to be used after upgrade
	FORMFACTOR=`/opt/ad/bin/CameraPersonalityDisplay --query FORMFACTOR`
	if [ "$FORMFACTOR" == "PTZ" ]
	then
	    # Save current pan, tilt & zoom position                                     
	    /opt/ad/bin/ptzposition -c 0 | /bin/grep Value > /opt/ad/etc/position.dat
	    logger -t $installer "saving last known ptz position"
	    sync
	fi

	destdir=/mnt/flash

    # Merge the old and new config server files before any upgrading occurs and keep them seperate in /tmp/config_merge
    # Without this any new paths in the new files won't be created as the new config is overwritten when the old partition config is copied across 

    # Old firmware versions may have the old osd_config.conf filename. Therefore we will always need to migrate it.

    if [ -f /opt/ad/etc/conf/osd_config.conf ]
    then
        cp /opt/ad/etc/conf/osd_config.conf /opt/ad/etc/conf/osd_config.values # Migrate osd_config to the new filename
    fi

    cp /opt/ad/etc/conf/*.values /mnt/flash/tmp/

    chroot /mnt/flash /bin/bash -c "mkdir -p /tmp/config_merge"
    chroot /mnt/flash /bin/bash -c "/opt/ad/bin/config_merger /tmp/ /opt/ad/etc/conf/ /tmp/config_merge/" 

	# make a temporary copy of the new PLATMAN.conf
	if [ -f "${destdir}/opt/ad/etc/conf/PLATMAN.conf" ];
	then
	    cp ${destdir}/opt/ad/etc/conf/PLATMAN.conf /tmp/PLATMAN.conf
	fi

	# make a temporary copy of the new CONFIGSERVER.cfg
	if [ -f "${destdir}/opt/ad/etc/conf/CONFIGSERVER.cfg" ];
	then
	    cp ${destdir}/opt/ad/etc/conf/CONFIGSERVER.cfg /tmp/CONFIGSERVER.cfg
	fi

    merge_files

	# Backup camera block settings to ensure CAMBLKSETTINGS.conf contains the latest settings
	# also restart onvif server to flush mediaconfiguration.db ( eg streams configuration ) to disk
	/opt/ad/bin/camerablocksettings backup
	sv restart onvifserver &> /dev/null

	# Stop faultmgr which will back up any faultmgr.db changes to flash.
	sv stop faultmgr

	#Copy config files from old partition to /mnt/flash
    for f in `grep ^ifexist:. /mnt/flash/opt/ad/etc/conf/CONFIGBK.conf | sed s/ifexist:./\\/g` ; do
        if [ -f $f ] ; then
            echo "Copying file: $f"
            cp  $f $destdir$f
        fi
        if [ -d $f ] ; then
            echo "Copying directory: $f"
            cp --parents -rf $f $destdir
        fi
    done
	for f in `egrep -v "^ifexist|^#" /mnt/flash/opt/ad/etc/conf/CONFIGBK.conf | sed s\&^.\&\&g`; do
        if [ -f $f ] ; then
            echo "Copying file: $f"
            cp $f $destdir$f
        fi
        if [ -d $f ] ; then
            echo "Copying directory: $f"
            cp --parents -rf $f $destdir
        fi
    done

    # IMPORTANT - ensure the legacy file named osd_config.conf does not get migrated as this could
    # confused future updates.

    if [ -e  /mnt/flash/opt/ad/etc/conf/osd_config.conf ]
    then
       rm -f /mnt/flash/opt/ad/etc/conf/osd_config.conf
    fi

    #As CAMBLKSETTINGS.conf doesn't ship with the new image it needs specifically copied across
    #Removing this line will result in camera block settings being defaulted on an upgrade
    cp /opt/ad/etc/conf/CAMBLKSETTINGS.conf /mnt/flash/opt/ad/etc/conf/CAMBLKSETTINGS.conf

	# always install the new PLATMAN.conf
	if [ -f "/tmp/PLATMAN.conf" ];
	then
	    cp /tmp/PLATMAN.conf ${destdir}/opt/ad/etc/conf/PLATMAN.conf
	    rm /tmp/PLATMAN.conf
	fi

	if [ -f "/tmp/CONFIGSERVER.cfg" ];
	then
	    cp /tmp/CONFIGSERVER.cfg ${destdir}/opt/ad/etc/conf/CONFIGSERVER.cfg
	    rm /tmp/CONFIGSERVER.cfg
	fi

	# Restore camerablocksettings now camerablocksettings are in place
	/opt/ad/bin/camerablocksettings restore

    #Copy merged/copied config files back to where they should be and cleanup /tmp
    cp /mnt/flash/tmp/config_merge/* /mnt/flash/opt/ad/etc/conf/
	rm -f /mnt/flash/opt/ad/etc/conf/*.definition
	rm -f /mnt/flash/opt/ad/etc/conf/*.personality
    rm -rf /mnt/flash/tmp/config_merge
    rm -f /mnt/flash/tmp/*.values

}

sub_upgrades()
{
    # Upgrades can depend on the controller running, so shut it down
    log "Sub upgrades: controllerTV shutting down..."
    sv stop controllerTV
    chroot /mnt/flash /bin/bash -c "/opt/ad/bin/mcu_firmware.sh --upgrade"
    chroot /mnt/flash /bin/bash -c "/opt/ad/bin/lens_firmware.sh --upgrade"
}

# YT: not sure if this belong here or in factory programming.  Place here for now.
initial_setup()
{
    # First generate defaults in the destination FS

    log "initialising configuration in new bank .. "
    chroot /mnt/flash /bin/bash -c "/opt/ad/bin/pstore_integrity_checker /opt/ad/etc/conf BACKUP_ONLY"

    partnum=`chroot /mnt/flash /bin/bash -c "/opt/ad/bin/pstorectl --print --field part_number"`

    chroot /mnt/flash /bin/bash -c "/opt/ad/bin/CameraPersonalityInit $partnum"		# Initialize camera personality
    chroot /mnt/flash /bin/bash -c "/opt/ad/bin/configtool -i"		# Install personalities in the new bank
    chroot /mnt/flash /bin/bash -c "/opt/ad/bin/configtool -d"		# Install default values in the new bank

    log "initialising configuration in new bank done "
    # Generate "factory defaults".

	/mnt/flash/opt/ad/bin/config_factory_create.sh
}

##################### Begin Script Operation ################
# output current firwmare version for information purpose

log "Upgrade : to image $1"

exec 200>/tmp/installer.lock
if ! flock -n 200  ; then
   echo "another instance is running";
   logger -s -t $installer -p user.err "Error: installer already running"
   exit 1
fi

# Direct output from remainder of this script

log "Firmware upgrade started at : "`date`

if [ 1 -ne $# ]; then
    fatal Usage: $0 "<firmware filename>"
    exit 1
fi

firmware=$1
if [ ! -f $firmware ]; then
    fatal "firmware file $firmware doesn't exist"
fi

# Detect which bank is currently active
if [ `fw_printenv activeimage` == "activeimage=A" ] ;
then
  log "Active Image is A"
  # destination is B (KL2, FS2)
  NEW_KERNEL_MTD=/dev/mtd7
  NEW_FS_MTD=/dev/mtd8
  NEW_MTD_NO=8
  NEW_FS_MTDBLOCK=/dev/mtdblock8
else if [ `fw_printenv activeimage` == "activeimage=B" ] ;
then
  log "Active Image is B"
  # destination is A (KL1, FS1)
  NEW_KERNEL_MTD=/dev/mtd5
  NEW_FS_MTD=/dev/mtd6
  NEW_MTD_NO=6
  NEW_FS_MTDBLOCK=/dev/mtdblock6
else
  fatal "unable to determine current active image"
fi
fi

# Clean up any previous upgrade operations.
# umount /mnt/flash if it is already mounted.

if `cat /proc/mounts |grep -q "/mnt/flash"`
then
	umount /mnt/flash
fi

if [ -e /dev/ubi1 ]
then
	busybox ubidetach -d 1 /dev/ubi_ctrl
fi

log "Extracting kernel"
#extract the kernel

############### UBI Install #####################
log "Erase kernel in ${NEW_KERNEL_MTD}"
flash_eraseall ${NEW_KERNEL_MTD} 		|| fatal "erasing flash partition ${NEW_KERNEL_MTD}"
log "Program kernel in ${NEW_KERNEL_MTD}"
tar zpxf ${firmware} fw.bin --to-stdout | /opt/ad/bin/config_decrypt.sh | tar zpxf - zImage --to-stdout | nandwrite -p ${NEW_KERNEL_MTD} || fatal "writing new kernel"

log "Attach new UBI : ${NEW_MTD_NO}"
mkdir -p /mnt/flash                     	|| fatal "creating directory /mnt/flash"
busybox ubiattach /dev/ubi_ctrl -m ${NEW_MTD_NO}	|| fatal "ubiattach partition ${NEW_MTD_NO}"

UBI1DEV=/dev/ubi1

log "Get UBI image size .. "

# Get the size of the UBIFS image. 
image_size=`tar -O -zpxf ${firmware} fw.bin |/opt/ad/bin/config_decrypt.sh|tar -tvz ubifs.img |awk '{print $3;}'`

log "Program image size $image_size to UBI /dev/ubi1_0 .. "

# Now program it .. 
tar -O -zpxf ${firmware} fw.bin |/opt/ad/bin/config_decrypt.sh|tar -O -zpxf - ubifs.img |busybox ubiupdatevol /dev/ubi1_0 -s $image_size

log "Program complete"

############### Explode the new filesystem image #####################

# Mount it first

log "Mount the new UBI in /mnt/flash"
mount -t ubifs ubi1:rootfs /mnt/flash

# bind-mount system fileystems on the other bank. This allows syslog & other services to work following a chroot.

mount --bind /dev /mnt/flash/dev/
mount --bind /proc /mnt/flash/proc/
mount --bind /sys /mnt/flash/sys/

############### initialise some device specific items###############

log "Begin initial setup"
initial_setup


log "Copy configuration"

# Copy config from old partition
copy_config

log "Setup apps"
# create user and group app:apps if not there already
/mnt/flash/opt/ad/bin/set_ug_app-apps.sh /mnt/flash

# setup the etc directory if needed
log "Setup etc"
/mnt/flash/opt/ad/bin/etc_scan.sh /mnt/flash

# Copy the calibration data.

log "Copy calibration data"
if [ -e /mnt/flash/opt/ad/bin/tv_cal_ctl ]
then
	/mnt/flash/opt/ad/bin/tv_cal_ctl --copy /mnt/flash
fi

sub_upgrades

log "Unmount new bank"
############### Unmount 'new' file system image #####################
cd /tmp

# Unmount the bind-mounts first.
umount /mnt/flash/dev/
umount /mnt/flash/proc/
umount /mnt/flash/sys/

umount /mnt/flash
rmdir /mnt/flash

############### Swap active image on next boot & Reboot #####################
fw_setenv swap_image 1                        || fatal "unable to swap images"
log "Rebooting now"

reboot
